#!/bin/sh

srcdir=$1
fasta=$srcdir"/seqs.fasta"
tree=$srcdir"/tree"

tgtdir=$2
aligstmp=$tgtdir"/aligstmp/"  # temporary directory
aligs=$tgtdir"/aligs/"
outdir=$tgtdir"/partitions/"
log=$tgtdir/LOG

##################### Test if everything is available #####################
test=1
[ -z $1 ] && { test=''; }
[ -z $2 ] && { test=''; }
[ -z $test ] && { 
echo "============================================================"
echo "Usage"
echo "------------------------------------------------------------"
echo "$0 <source-directory> <target-directory>"
echo ""
echo "source-directory must contain:"
echo " seqs.fasta 	input sequences of cluster pipeline in FASTA format"
echo " tree       	hierarchical cluster tree in NEWICK format"
exit; 
}

test=1
[ ! -d $1 ] && { echo "Source directory not found."; test=''; }
[ ! -f $fasta ] && { echo "Source directory '$srcdir' does not contain file 'seqs.fasta'"; test=''; } 
[ ! -f $tree ] && { echo "Source directory '$srcdir' does not contain file 'tree'"; test=''; }
[ -z $test ] && exit;

echo "============================================================"
echo "Check for required binaries"
echo "------------------------------------------------------------"
test=1
printf "mlocarna ... "
[ -z `which mlocarna 2>/dev/null` ] && { echo "'mlocarna' is not installed."; test=''; } || echo "ok"
printf "locarna ... "
[ -z `which locarna 2>/dev/null` ] && { echo "'locarna' is not installed."; test=''; } || echo "ok"
printf "RNAalifold ... "
[ -z `which RNAalifold 2>/dev/null` ] && { echo "'RNAalifold' is not installed (see Vienna RNA package)."; test=''; } || echo "ok"
printf "coloraln.pl ... "
[ -z `which coloraln.pl 2>/dev/null` ] && { echo "'coloraln.pl' is not installed (see Vienna RNA package)."; test=''; } || echo "ok"
printf "rnasoup_consMFE.pl ... "
[ -z `which rnasoup_consMFE.pl 2>/dev/null` ] && { echo "'rnasoup_consMFE.pl' is not installed."; test=''; } || echo "ok"
printf "RNAsoup ... "
[ -z `which RNAsoup 2>/dev/null` ] && { echo "'RNAsoup' is not installed."; test=''; } || echo "ok"
printf "perl ... "
[ -z `which perl 2>/dev/null` ] && { echo "'perl' is not installed."; test=''; } || echo "ok"
printf "convert ... "
[ -z `which perl 2>/dev/null` ] && { echo "'convert' is not installed."; test=''; } || echo "ok"
printf "rename ... "
[ -z `which perl 2>/dev/null` ] && { echo "'rename' is not installed."; test=''; } || echo "ok"
[ -z $test ] && exit;

##################### Progressive multiple alignments for all nodes within the tree #####################
mkdir -p $aligstmp;
mkdir -p $aligs;
echo "============================================================"
echo "Traverse tree and compute multiple alignments progressively"
echo "------------------------------------------------------------"
echo "============================================================" > $log
echo "Traverse tree and compute multiple alignments progressively"  >> $log
echo "------------------------------------------------------------" >> $log
path_tree=`(cd "$(dirname "$tree")"; pwd)`
# There is a small bug in mlocarna - it reports a strange error in case tree ends with ';'
perl -ne '$_=~s/;//; print $_;' $path_tree/tree > $path_tree/.tree 2>>$log
mlocarna --treefile $path_tree/.tree --tgtdir $aligstmp $fasta > $tgtdir/mlocarna.out 2>>$log;
tree=`basename $tree`
tree=$path_tree/tree

## Keep only intermediate alignments 
for i in $aligstmp/intermediate*.aln;
do mv ${i} $aligs; done;
rm -rf $aligstmp;

##################### Create mfe_consensus.txt #########################################################
echo "============================================================"
echo "Call RNAalifold -color -d2 -noLP for all internal nodes with"
echo "less than 500 leaves"
echo "------------------------------------------------------------"
echo "============================================================" >> $log
echo "Call RNAalifold -color -d2 -noLP for all internal nodes with" >> $log
echo "less than 500 leaves"					    >> $log
echo "------------------------------------------------------------" >> $log
rnasoup_consMFE.pl -f $fasta -d $aligs > $tgtdir/mfe_consensus.txt  2>>$log

##################### Call RNAsoup #####################
echo "============================================================"
echo "Call RNAsoup - Spot grOUPs in RNA cluster-tree"
echo "------------------------------------------------------------"
echo "============================================================"  >> $log
echo "Call RNAsoup - Spot grOUPs in RNA cluster-tree"		     >> $log
echo "------------------------------------------------------------"  >> $log
perl -ne 'if($_!~/;/){ chomp $_; print $_,";\n";}else{ print $_; }' $tree > $tgtdir/.tree 2>>$log
RNAsoup -t $tgtdir/.tree -f $fasta -m $tgtdir/mfe_consensus.txt -o $outdir 2>>$log

##################### Convert PS pictures to GIF #####################
echo "============================================================"  >> $log
echo "Convert PS pictures to GIF (needed for SoupViewer)"            >> $log
echo "------------------------------------------------------------"  >> $log
for i in $aligs/*.ps; do convert ${i} ${i}.gif; done;
for i in $aligs/*.ps.gif; do mv ${i} ${i/.ps/""}; done;

error=`grep -i error $log`
[ -n "$error" ] && { 
echo "------------------------------------------------------------";
echo "Script exit abnormally - please check $log.";
echo "------------------------------------------------------------";
}

echo "------------------------------------------------------------"
echo "Cheers."
echo "------------------------------------------------------------"
